@@ -26,7 +26,6 @@
 #include <gladeui/glade-editor.h>
 #include <gladeui/glade-palette.h>
 #include <gladeui/glade-clipboard.h>
-#include <gladeui/glade-project-view.h>
 
 G_BEGIN_DECLS
 
@@ -37,13 +36,32 @@ G_BEGIN_DECLS
 #define GLADE_IS_APP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GLADE_TYPE_APP))
 #define GLADE_APP_GET_CLASS(o)    (G_TYPE_INSTANCE_GET_CLASS ((o), GLADE_APP, GladeAppClass))
 
-typedef struct _GladeApp        GladeApp;
-typedef struct _GladeAppPrivate GladeAppPrivate;
-typedef struct _GladeAppClass   GladeAppClass;
+#define GLADE_TYPE_POINTER_MODE   (glade_pointer_mode_get_type())
+
+typedef struct _GladeApp         GladeApp;
+typedef struct _GladeAppPrivate  GladeAppPrivate;
+typedef struct _GladeAppClass    GladeAppClass;
+typedef enum   _GladePointerMode GladePointerMode;
+
+
+/**
+ * GladePointerMode:
+ * @GLADE_POINTER_SELECT:      Mouse pointer used for selecting widgets
+ * @GLADE_POINTER_ADD_WIDGET:  Mouse pointer used for adding widgets
+ * @GLADE_POINTER_DRAG_RESIZE: Mouse pointer used for dragging and 
+ *                             resizing widgets in containers
+ *
+ * Indicates what the pointer is used for in the workspace.
+ */
+enum _GladePointerMode
+{
+	GLADE_POINTER_SELECT = 0,
+	GLADE_POINTER_ADD_WIDGET,
+	GLADE_POINTER_DRAG_RESIZE
+};
 
 struct _GladeApp
 {
-	/*< private >*/
 	GObject parent_instance;
 	
 	GladeAppPrivate *priv;
@@ -59,22 +77,20 @@ struct _GladeAppClass
 	void   (*  hide_properties) (GladeApp* app);
 
 	/* signals */
-	void   (* widget_event)     (GladeApp    *app, 
-				     GladeWidget *toplevel,
-				     GdkEvent    *event);
 	void   (* update_ui_signal) (GladeApp    *app);
 };
 
  
 GType              glade_app_get_type   (void) G_GNUC_CONST;
 
+GladeApp*          glade_app_new        (void);
+
+GType              glade_pointer_mode_get_type (void) G_GNUC_CONST;
+
 GladeApp*          glade_app_get        (void);
 
 void               glade_app_update_ui  (void);
 
-gboolean           glade_app_widget_event (GladeWidget *widget, 
-					   GdkEvent    *event);
- 
 void               glade_app_set_window (GtkWidget *window);
  
 GtkWidget*         glade_app_get_window (void);
@@ -88,11 +104,14 @@ GladeClipboard*    glade_app_get_clipboard (void);
  
 GtkWidget*         glade_app_get_clipboard_view (void);
 
- 
 GladeProject*      glade_app_get_project (void);
  
 void               glade_app_set_project (GladeProject *project);
+
+GladePointerMode   glade_app_get_pointer_mode (void);
  
+void               glade_app_set_pointer_mode (GladePointerMode mode);
+
 void               glade_app_add_project (GladeProject *project);
  
 void               glade_app_remove_project (GladeProject *project);
@@ -110,9 +129,6 @@ void               glade_app_show_properties (gboolean raise);
 void               glade_app_hide_properties (void);
 
  
-void               glade_app_add_project_view (GladeProjectView *view);
-
- 
 void               glade_app_command_copy (void);
  
 void               glade_app_command_cut (void);
@@ -178,8 +194,6 @@ const gchar       *glade_app_get_pixmaps_dir   (void) G_GNUC_CONST;
 
 const gchar       *glade_app_get_locale_dir    (void) G_GNUC_CONST;
 
-const gchar       *glade_app_get_bindings_dir  (void) G_GNUC_CONST;
-
 G_END_DECLS
 
 #endif /* __GLADE_APP_H__ */
